#if !defined(_CIW_GAME_UTIL_H_)
#define _CIW_GAME_UTIL_H_

#include "s3e.h"

#ifndef NULL
    #define NULL 0
#endif

#define SAFE_DELETE(x) if (x != NULL) { delete x; x = NULL; }
#define SAFE_DELETE_ARRAY(x) if (x != NULL) { delete [] x; x = NULL; }

//
//
// SINGLETONS
//
//
//
// Define a class as a singleton (Add to class definition in header file)
//
#define CDEFINE_SINGLETON(the_class)				\
private:										\
	static the_class* _instance;				\
	the_class() {}								\
	~the_class() {}								\
	the_class(const the_class &);				\
	the_class& operator=(const the_class &);	\
public:											\
	static void Create();						\
	static void Destroy();						\
	static the_class* getInstance();

//
// Declare singleton methods (Add to source file)
//
#define CDECLARE_SINGLETON(the_class)		\
the_class* the_class::_instance = NULL;		\
void the_class::Create()					\
{											\
	if (_instance == NULL)					\
		_instance = new the_class;			\
}											\
void the_class::Destroy()					\
{											\
	if (_instance != NULL)					\
	{										\
		delete _instance;					\
		_instance = NULL;					\
	}										\
}											\
the_class* the_class::getInstance()			\
{											\
  return _instance;							\
}

//
//
// CIwGameError - Game error logging class
//
//
class CIwGameError
{
public:
	static void LogError(const char* message);
	static void LogError(const char* message, const char* data);
};

//
//
// CIwGameCallback - Game callback type
//
//
typedef int32 (*CIwGameCallback)(void* caller);

//
//
// CIwGameTimer (passive - needs to be polled with HasTimedOut)
//
//
class CIwGameTimer
{
protected:
	bool		Started;			// true if timer started
	uint64 		LastTime;			// Time that the timer was set (milliseconds)

	// Properties
private:
	uint64		Duration;			// Duration of timer in milliseconds
	bool		AutoReset;			// Auto reset the timer when it times out
public:
	void		setDuration(uint64 millseconds_duration, bool start = true)
	{
		Duration = millseconds_duration;
		if (start)
			Start();
	}
	uint64		getDuration() const				{ return Duration; }
	void		setAutoReset(bool auto_reset)	{ AutoReset = auto_reset; }
	bool		getAutoReset() const			{ return AutoReset; }
	bool		hasStarted() const				{ return Started; }
	bool		hasStopped() const				{ return !Started; }
	// Properties end

public:
	CIwGameTimer() : LastTime(0), Duration(0), AutoReset(false), Started(false) { }
	CIwGameTimer(int millseconds_duration)
	{
		LastTime = 0;
		Duration = 0;
		AutoReset = false;
		Started = false;
		setDuration(millseconds_duration);
	}
	~CIwGameTimer() {	}
	
	virtual bool HasTimedOut()
	{
		if (!Started)
			return false;

		if (Duration == 0)	// Timer of 0 duration never times out
		{
			return false;
		}
		
		uint64 time_diff = GetElapsedTime();
	
		if (time_diff > Duration)
		{
			if (AutoReset)
			{
				LastTime = s3eTimerGetMs();
			}
			else
				Started = false;
			return true;
		}
	
		return false;
	}
	uint64		GetElapsedTime() const
	{
		return s3eTimerGetMs() - LastTime;
	}
	uint64		GetTimeDiff(uint64 this_time) const { return this_time - LastTime; }
	uint64		GetTimeDiff() const					{ return s3eTimerGetMs() - LastTime; }
	uint64		GetTimeLeft() const					{ return Duration - (s3eTimerGetMs() - LastTime); }
	void		Reset()								{ setDuration(Duration); }
	void		Start()
	{
		LastTime = s3eTimerGetMs();
		Started = true;
	}
	void		Stop()								{ Started = false; }

};


#endif	// _CIW_GAME_UTIL_H_
