// Marmalade headers
#include "s3e.h"
#include "Iw2D.h"
#include "IwGx.h"
#include "IwResManager.h"
#include "IwSound.h"

// iwGame engine headers
#include "IwGameInput.h"
#include "IwGameAnim.h"
#include "IwGameScene.h"
#include "IwGameCamera.h"

// Game headers
#include "Common.h"
#include "ActorPlayer.h"

int main()
{
	// Initialise Mamrlade graphics system and Iw2D module
	IwGxInit();
    Iw2DInit();

	// Init IwSound
	IwSoundInit();

	// Initialise the rsource manager
	IwResManagerInit();

#ifdef IW_BUILD_RESOURCES
	// Tell resource system how to convert WAV files
	IwGetResManager()->AddHandler(new CIwResHandlerWAV);
#endif

	// Load our Level1 resource group into the resource manager
	IwGetResManager()->LoadGroup("Level1.group");

	// Initialise the input system
	CIwGameInput::Create();
	IW_GAME_INPUT->Init();

	// Set the default background clear colour
	IwGxSetColClear(0x40, 0x40, 0x40, 0);

	// Create two images from PNG resource files
	CIwResGroup* Level1Group = IwGetResManager()->GetGroupNamed("Level1");
	IwGetResManager()->SetCurrentGroup(Level1Group);		// Ensure that Level1 is the current resource group
	CIw2DImage* sprites_image = Iw2DCreateImageResource("sprites");

	// Get the display surface width and height
	int surface_width = Iw2DGetSurfaceWidth();
	int surface_height = Iw2DGetSurfaceHeight();
	int	max_size = (surface_width > surface_height) ? surface_width / 2: surface_height / 2;

	// Create a scene
	CIwGameScene* game_scene = new CIwGameScene();
	game_scene->Init();
	game_scene->setVirtualTransform(VIRTUAL_SCREEN_WIDTH, VIRTUAL_SCREEN_HEIGHT, 0, true, false);

	// Create a camera and attach it to the scene
	CIwGameCamera* game_camera = new CIwGameCamera();
	game_scene->setCamera(game_camera);

	// Allocate animation frames for our player
	CIwGameAnimImageFrame* anim_frames = game_scene->getAnimFrameManager()->allocImageFrames(8, 36, 40, 0, 0, 512, 40, 512);

	// Dynamic variables
	bool		PrevTouched = false;

	// Play some MP3 music using s3e Audio (if the codec is supported)
//	if (s3eAudioIsCodecSupported(S3E_AUDIO_CODEC_MP3))
//		s3eAudioPlay("music.mp3", 1);

	// Main Game Loop
	while (!s3eDeviceCheckQuitRequest())
	{
		// Clear the screen
		IwGxClear(IW_GX_COLOUR_BUFFER_F | IW_GX_DEPTH_BUFFER_F);

		// Update pointer system
		IW_GAME_INPUT->Update();

		// Update keyboard system
		if (IW_GAME_INPUT->isKeyDown(s3eKeyAbsBSK))	// Back key is used to exit on some platforms
			break;

		// Update Iw Sound Manager
		IwGetSoundManager()->Update();

		if (IW_GAME_INPUT->getTouchCount() > 0)
		{
			if (!PrevTouched)
			{
				// Get tapped position in our virtual screen space
				CIwFVec2 pos = game_scene->ScreenToVirtual(IW_GAME_INPUT->getTouch(0)->x, IW_GAME_INPUT->getTouch(0)->y);

				// Create 10 player actors
				for (int t = 0; t < 10; t++)
				{
					// Create and set up our face animation
					CIwGameAnimImage* face_anim = new CIwGameAnimImage();
					face_anim->setFrameData(anim_frames, 8);
					face_anim->setLooped(-1);
					face_anim->setPlaybackSpeed(0.2f);
					face_anim->Start();

					// Create player actor
					ActorPlayer* player = new ActorPlayer();
					player->Init(game_scene, sprites_image, face_anim, 36, 40);
					player->setName("Player");
					player->setPosition(pos.x, pos.y);

					// Add player actor to the scene
					game_scene->addActor(player);
				}
			}
			PrevTouched = true;
		}
		else
			PrevTouched = false;

		// Update the scene
		game_scene->Update(1.0f);

		// Draw the scene
		game_scene->Draw();

		// Show the surface
		Iw2DSurfaceShow();

		// Yield to the operating system
		s3eDeviceYield(0);
	}

	// Safely clean up the camera
	if (game_camera != NULL)
		delete game_camera;

	// Safely cleanup game scene
	if (game_scene != NULL)
		delete game_scene;

	// Safely cleanup images
	if (sprites_image != NULL)
		delete sprites_image;

	// Shut down the input system
	IW_GAME_INPUT->Release();
	CIwGameInput::Destroy();

	// Shut down the resource manager
	IwResManagerTerminate();

	// Shutdown IwSound
	IwSoundTerminate();

	// Shut down Marmalade graphics system and the Iw2D module
	Iw2DTerminate();
	IwGxTerminate();

    return 0;
}

